export declare module EventTypes {
    interface IBigNumber {
        toNumber: () => number;
    }
    interface IEsEventFromTruffle {
        Id: IBigNumber;
        Type: string;
        Version: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: IBigNumber;
        Bytes32Value: string;
        TxOrigin: string;
        Created: IBigNumber;
        PropertyCount: IBigNumber;
    }
    interface IEsEventPropertyFromTruffle {
        EventIndex: IBigNumber;
        EventPropertyIndex: IBigNumber;
        Name: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: IBigNumber;
        Bytes32Value: string;
    }
    interface IEsEvent {
        Id?: number;
        Type: string;
        Version: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: number;
        Bytes32Value: string;
        TxOrigin?: string;
        Created?: number;
        PropertyCount?: number;
    }
    interface IEsEventProperty {
        EventIndex: number;
        EventPropertyIndex: number;
        Name: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: number;
        Bytes32Value: string;
    }
    interface ITransmuteEventMeta {
        id: number;
        version: string;
        txOrigin: string;
        created: number;
    }
    interface ITransmuteEvent {
        type: string;
        payload: any;
        error?: any;
        meta: ITransmuteEventMeta;
    }
    interface ITransmuteCommand {
        type: string;
        payload: any;
    }
    interface ITransaction {
        tx: string;
        receipt: any;
        logs: any[];
    }
    interface ITransmuteCommandResponse {
        events: Array<ITransmuteEvent>;
        transactions: Array<ITransaction>;
    }
    interface IReadModel {
        lastEvent: number;
        readModelType: string;
        readModelStoreKey: string;
        contractAddress: string;
        model: any;
    }
    const toAscii: (value: any) => any;
    const guessType: (value: any) => "Address" | "UInt" | "Bytes32" | "Object";
    const EsEvent = "EsEvent";
    const EsEventProperty = "EsEventProperty";
    const EsEventPropertySchema: {
        EventIndex: string;
        EventPropertyIndex: string;
        Name: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: string;
        Bytes32Value: string;
    };
    const EsEventSchema: {
        Id: string;
        Type: string;
        Version: string;
        ValueType: string;
        AddressValue: string;
        UIntValue: string;
        Bytes32Value: string;
        TxOrigin: string;
        Created: string;
        PropertyCount: string;
    };
    const TruffleEventSchema: {
        [x: string]: {
            EventIndex: string;
            EventPropertyIndex: string;
            Name: string;
            ValueType: string;
            AddressValue: string;
            UIntValue: string;
            Bytes32Value: string;
        } | {
            Id: string;
            Type: string;
            Version: string;
            ValueType: string;
            AddressValue: string;
            UIntValue: string;
            Bytes32Value: string;
            TxOrigin: string;
            Created: string;
            PropertyCount: string;
        };
    };
    const getPropFromTruffleEventSchemaType: (schemaPropType: any, truffleValue: any) => any;
    const getTransmuteEventPropertyFromEsEventProperty: (prop: any) => {};
    const getEsEventFromEsEventValues: (eventValues: any) => IEsEventFromTruffle;
    const getEsEventPropertyFromEsEventPropertyValues: (eventPropValues: any) => IEsEventPropertyFromTruffle;
    const getEsEventFromEsEventWithTruffleTypes: (eventType: string, solEvent: IEsEventFromTruffle) => IEsEvent;
    const getEsEventPropertyFromEsEventPropertyWithTruffleTypes: (eventType: string, solEvent: IEsEventPropertyFromTruffle) => IEsEventProperty;
    const getValueType: (es: any) => any;
    const convertMeta: (esEvent: IEsEvent) => any;
    const payloadReducer: (state: any, esEvenProperty: IEsEventProperty) => any;
    const esEventToTransmuteEvent: (esEvent: IEsEvent, esEventProps?: IEsEventProperty[]) => Promise<ITransmuteEvent>;
    const convertCommandToEsEvent: (transmuteCommand: ITransmuteCommand) => IEsEvent;
    const convertCommandToEsEventProperties: (esEvent: IEsEvent, transmuteCommand: ITransmuteCommand) => IEsEventProperty[];
    const flatten: (data: any) => {};
    const unflatten: (data: Object) => Object;
    const eventsFromTransaction: (tx: any) => any;
    const reconstructTransmuteEventsFromTxs: (txs: ITransaction[]) => any[];
}
