"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const { keys, pick, omit, flatten, difference, extend } = require('lodash');
const EventTypes_1 = require("../EventTypes/EventTypes");
const _ = require("lodash");
const flux_standard_action_1 = require("flux-standard-action");
let DEBUG = true;
var Middleware;
(function (Middleware) {
    Middleware.writeEsEvent = (eventStore, fromAddress, esEvent) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let { Type, Version, ValueType, AddressValue, UIntValue, Bytes32Value, PropertyCount } = esEvent;
        return yield eventStore.writeEvent(Type, Version, ValueType, AddressValue, UIntValue, Bytes32Value, PropertyCount, {
            from: fromAddress,
            gas: 2000000
        });
    });
    Middleware.writeEsEventProperty = (eventStore, fromAddress, esEventProp) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let { EventIndex, EventPropertyIndex, Name, ValueType, AddressValue, UIntValue, Bytes32Value } = esEventProp;
        return yield eventStore.writeEventProperty(EventIndex, EventPropertyIndex, Name, ValueType, AddressValue, UIntValue, Bytes32Value, {
            from: fromAddress,
            gas: 2000000
        });
    });
    Middleware.readEsEventValues = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        return yield eventStore.readEvent.call(eventId, {
            from: fromAddress,
            gas: 2000000
        });
    });
    Middleware.readEsEventPropertyValues = (eventStore, fromAddress, eventId, propId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        return yield eventStore.readEventProperty.call(eventId, propId, {
            from: fromAddress,
            gas: 2000000
        });
    });
    Middleware.readEsEventPropertiesFromEsEvent = (eventStore, fromAddress, esEvent) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let eventId = esEvent.Id;
        let propId = 0;
        let promises = [];
        while (propId < esEvent.PropertyCount) {
            let eventPropVals = yield Middleware.readEsEventPropertyValues(eventStore, fromAddress, eventId, propId);
            let esEventPropWithTruffleTypes = EventTypes_1.EventTypes.getEsEventPropertyFromEsEventPropertyValues(eventPropVals);
            let esEventProp = EventTypes_1.EventTypes.getEsEventPropertyFromEsEventPropertyWithTruffleTypes('EsEventProperty', esEventPropWithTruffleTypes);
            promises.push(esEventProp);
            propId++;
        }
        return yield Promise.all(promises);
    });
    Middleware.readTransmuteEvent = (eventStore, fromAddress, eventId) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let eventVals = yield Middleware.readEsEventValues(eventStore, fromAddress, eventId);
        let esEventWithTruffleTypes = EventTypes_1.EventTypes.getEsEventFromEsEventValues(eventVals);
        let esEvent = EventTypes_1.EventTypes.getEsEventFromEsEventWithTruffleTypes('EsEvent', esEventWithTruffleTypes);
        let esEventProps = yield Middleware.readEsEventPropertiesFromEsEvent(eventStore, fromAddress, esEvent);
        let transmuteEvent = yield EventTypes_1.EventTypes.esEventToTransmuteEvent(esEvent, esEventProps);
        if (DEBUG && !flux_standard_action_1.isFSA(transmuteEvent)) {
            console.warn('WARNING: transmuteEvent: ', transmuteEvent, ' is not a FSA. see https://github.com/acdlite/flux-standard-action');
        }
        return transmuteEvent;
    });
    Middleware.readTransmuteEvents = (eventStore, fromAddress, eventId = 0) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let currentEvent = (yield eventStore.solidityEventCount()).toNumber();
        let eventPromises = [];
        while (eventId < currentEvent) {
            eventPromises.push(yield Middleware.readTransmuteEvent(eventStore, fromAddress, eventId));
            eventId++;
        }
        return yield Promise.all(eventPromises);
    });
    Middleware.writeTransmuteCommand = (eventStore, fromAddress, transmuteCommand) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let esEvent = EventTypes_1.EventTypes.convertCommandToEsEvent(transmuteCommand);
        let tx = yield Middleware.writeEsEvent(eventStore, fromAddress, esEvent);
        let eventsFromWriteEsEvent = yield EventTypes_1.EventTypes.eventsFromTransaction(tx);
        let esEventWithIndex = eventsFromWriteEsEvent[0];
        let esEventProperties = EventTypes_1.EventTypes.convertCommandToEsEventProperties(esEventWithIndex, transmuteCommand);
        let allTxs = [tx];
        if (esEventWithIndex.PropertyCount) {
            let esEventPropertiesWithTxs = esEventProperties.map((esp) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                return yield Middleware.writeEsEventProperty(eventStore, fromAddress, esp);
            }));
            let txs = yield Promise.all(esEventPropertiesWithTxs);
            allTxs = allTxs.concat(txs);
        }
        allTxs = _.flatten(allTxs);
        let transmuteEvents = yield Promise.all(EventTypes_1.EventTypes.reconstructTransmuteEventsFromTxs(allTxs));
        return {
            events: transmuteEvents,
            transactions: allTxs
        };
    });
    Middleware.writeTransmuteCommands = (eventStore, fromAddress, transmuteCommands) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        let promises = transmuteCommands.map((cmd) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            return yield Middleware.writeTransmuteCommand(eventStore, fromAddress, cmd);
        }));
        return yield Promise.all(promises);
    });
})(Middleware = exports.Middleware || (exports.Middleware = {}));
